/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package jsr308.annotations.on.java.types;

import java.io.File;
import java.lang.annotation.ElementType;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

@Target({ElementType.TYPE_PARAMETER, ElementType.TYPE_USE})
@interface NonNull{}

@Target(ElementType.TYPE_USE)
@interface Readonly{}

@Target(ElementType.TYPE_USE)
@interface Existing{}

@Target(ElementType.TYPE_USE)
@interface Long{}

@Target(ElementType.TYPE_USE)
@interface Critical{}

@Target(ElementType.TYPE_PARAMETER)
@interface Immutable{}

/** @author mstoynov */
public class JSR308Example2 {
    public static <T> T method(String s) {return null;}

    public static void main() {
        //for generic type arguments in a generic method or constructor invocation:
        JSR308Example2.<@NonNull String>method("...");

        //for type parameters and type parameter bounds and wildcard bounds:
        Collection<@Long ? super @Existing File> c = null;

        //for class inheritance:
        abstract class UnmodifiableList<T> implements @Readonly List<@Readonly T> { }
    }
  //for throws clauses:
  void monitorTemperature() throws @Critical Exception {}

  //for method receivers:
  public String toString() @Readonly { return null; }

}

//for constructor declaration return types:
class Invocation {
  @Immutable Invocation() { }
}

class Arrays {
  class Document{}

  //for arrays:
  @Readonly Document [][] docs1 = new @Readonly Document [2][12]; // array of arrays of read-only documents
  Document @Readonly [][] docs2 = new Document @Readonly [2][12]; // read-only array of arrays of documents
  Document[] @Readonly [] docs3 = new Document[2] @Readonly [12]; // array of read-only arrays of documents
}

//for typecasts:
//myString = (@NonNull String) myObject;
//It is not permitted to omit the Java type, as in myString = (@NonNull) myObject;; see Sections B.2 and D.2.4.
//for constructor invocation results:
//  new @Interned MyObject()
//For generic constructors (JLS §8.8.4), the annotation follows the explicit type arguments (JLS §15.9):
//
//  new <String> @Interned MyObject()
//for type tests:
//  boolean isNonNull = myString instanceof @NonNull String;
//It is not permitted to omit the Java type, as in myString instanceof @NonNull; see Sections B.2 and D.2.4.
//for object creation:
//  new @NonEmpty @Readonly List<String>(myNonEmptyStringSet)
//for class literals:
//  Class<@NonNull String> c = @NonNull String.class;
//for static member access:
//  @NonNull Type.field
